@@ -17,6 +17,16 @@
 #include "PBblas.h"
 
 #ifdef __STDC__
+#ifdef FC_LEN_T
+void pdatrmv_( F_CHAR_T UPLO, F_CHAR_T TRANS, F_CHAR_T DIAG, int * N,
+               double * ALPHA,
+               double * A, int * IA, int * JA, int * DESCA,
+               double * X, int * IX, int * JX, int * DESCX,
+               int * INCX, double * BETA,
+               double * Y, int * IY, int * JY, int * DESCY,
+               int * INCY,
+               FC_LEN_T UPLO_len, FC_LEN_T TRANS_len, FC_LEN_T DIAG_len )
+#else
 void pdatrmv_( F_CHAR_T UPLO, F_CHAR_T TRANS, F_CHAR_T DIAG, int * N,
                double * ALPHA,
                double * A, int * IA, int * JA, int * DESCA,
@@ -24,6 +34,7 @@ void pdatrmv_( F_CHAR_T UPLO, F_CHAR_T TRANS, F_CHAR_T DIAG, int * N,
                int * INCX, double * BETA,
                double * Y, int * IY, int * JY, int * DESCY,
                int * INCY )
+#endif
 #else
 void pdatrmv_( UPLO, TRANS, DIAG, N, ALPHA, A, IA, JA, DESCA, X, IX,
                JX, DESCX, INCX, BETA, Y, IY, JY, DESCY, INCY )
@@ -482,8 +493,14 @@ void pdatrmv_( UPLO, TRANS, DIAG, N, ALPHA, A, IA, JA, DESCA, X, IX,
 *  Computational partitioning size is computed as the product of the logical
 *  value returned by pilaenv_ and 2 * lcm( nprow, npcol )
 */
+#ifdef FC_LEN_T
+      nb = 2 * pilaenv_( &ctxt, C2F_CHAR( &utyp->type ),
+                         (FC_LEN_T) strlen(C2F_CHAR( &utyp->type )) ) *
+           PB_Clcm( ( Arow >= 0 ? nprow : 1 ), ( Acol >= 0 ? npcol : 1 ) );
+#else
       nb = 2 * pilaenv_( &ctxt, C2F_CHAR( &utyp->type ) ) *
            PB_Clcm( ( Arow >= 0 ? nprow : 1 ), ( Acol >= 0 ? npcol : 1 ) );
+#endif
 
       if( upper )
       {
@@ -497,10 +514,24 @@ void pdatrmv_( UPLO, TRANS, DIAG, N, ALPHA, A, IA, JA, DESCA, X, IX,
                Anq0 = PB_Cnumroc( kb, k, Ainb1, Anb, mycol, Acol, npcol );
                if( Akp > 0 && Anq0 > 0 )
                {
+#ifdef FC_LEN_T
+                  dagemv_( TRANS, &Akp, &Anq0, ((double *) ALPHA),
+                           (double*) Mptr( Aptr, 0, Akq,  Ald, size ), &Ald,
+                           (double*) Mptr( XA,   0, Akq, XAld, size ), &XAld, (double*) one,
+                           (double*) YA, &ione,
+                           (FC_LEN_T) strlen(TRANS) );
+#else
+/*WCC
                   dagemv_( TRANS, &Akp, &Anq0, ((char *) ALPHA),
                            Mptr( Aptr, 0, Akq,  Ald, size ), &Ald,
                            Mptr( XA,   0, Akq, XAld, size ), &XAld, one,
                            YA, &ione );
+*/
+                  dagemv_( TRANS, &Akp, &Anq0, ((double *) ALPHA),
+                           (double*) Mptr( Aptr, 0, Akq,  Ald, size ), &Ald,
+                           (double*) Mptr( XA,   0, Akq, XAld, size ), &XAld, (double*) one,
+                           (double*) YA, &ione );
+#endif
                }
                PB_Cptrm( type, utyp, LEFT, UPPER, &TranOp, &DiagA, kb, 1,
                          ((char *) ALPHA), Aptr, k, k, Ad0,
@@ -518,9 +549,21 @@ void pdatrmv_( UPLO, TRANS, DIAG, N, ALPHA, A, IA, JA, DESCA, X, IX,
                Anq0 = PB_Cnumroc( kb, k, Ainb1, Anb, mycol, Acol, npcol );
                if( Akp > 0 && Anq0 > 0 )
                {
+#ifdef FC_LEN_T
+                  dagemv_( TRANS, &Akp, &Anq0, ((double *) ALPHA),
+                           (double*) Mptr( Aptr, 0, Akq, Ald, size ), &Ald, (double*) XA, &ione,
+                           (double*) one, (double*) Mptr( YA, 0, Akq, YAld, usiz ), &YAld,
+                           (FC_LEN_T) strlen(TRANS) );
+#else
+/*WCC
                   dagemv_( TRANS, &Akp, &Anq0, ((char *) ALPHA),
                            Mptr( Aptr, 0, Akq, Ald, size ), &Ald, XA, &ione,
                            one, Mptr( YA, 0, Akq, YAld, usiz ), &YAld );
+*/
+                  dagemv_( TRANS, &Akp, &Anq0, ((double *) ALPHA),
+                           (double*) Mptr( Aptr, 0, Akq, Ald, size ), &Ald, (double*) XA, &ione,
+                           (double*) one, (double*) Mptr( YA, 0, Akq, YAld, usiz ), &YAld );
+#endif
                }
                PB_Cptrm( type, utyp, LEFT, UPPER, &TranOp, &DiagA, kb, 1,
                          ((char *) ALPHA), Aptr, k, k, Ad0,
@@ -547,10 +590,24 @@ void pdatrmv_( UPLO, TRANS, DIAG, N, ALPHA, A, IA, JA, DESCA, X, IX,
                Anq0 = PB_Cnumroc( kb,   k, Ainb1, Anb, mycol, Acol, npcol );
                if( Amp0 > 0 && Anq0 > 0 )
                {
+#ifdef FC_LEN_T
+                  dagemv_( TRANS, &Amp0, &Anq0, ((double *) ALPHA),
+                           (double*) Mptr( Aptr, Akp, Akq,  Ald, size ), &Ald,
+                           (double*) Mptr( XA,     0, Akq, XAld, size ), &XAld, (double*) one,
+                           (double*) Mptr( YA,   Akp,   0, YAld, usiz ), &ione,
+                           (FC_LEN_T) strlen(TRANS) );
+#else
+/*WCC
                   dagemv_( TRANS, &Amp0, &Anq0, ((char *) ALPHA),
                            Mptr( Aptr, Akp, Akq,  Ald, size ), &Ald,
                            Mptr( XA,     0, Akq, XAld, size ), &XAld, one,
                            Mptr( YA,   Akp,   0, YAld, usiz ), &ione );
+*/
+                  dagemv_( TRANS, &Amp0, &Anq0, ((double *) ALPHA),
+                           (double*) Mptr( Aptr, Akp, Akq,  Ald, size ), &Ald,
+                           (double*) Mptr( XA,     0, Akq, XAld, size ), &XAld, (double*) one,
+                           (double*) Mptr( YA,   Akp,   0, YAld, usiz ), &ione );
+#endif
                }
             }
          }
@@ -570,10 +627,24 @@ void pdatrmv_( UPLO, TRANS, DIAG, N, ALPHA, A, IA, JA, DESCA, X, IX,
                Anq0 = PB_Cnumroc( kb,   k, Ainb1, Anb, mycol, Acol, npcol );
                if( Amp0 > 0 && Anq0 > 0 )
                {
+#ifdef FC_LEN_T
+                  dagemv_( TRANS, &Amp0, &Anq0, (double*) one,
+                           (double*) Mptr( Aptr, Akp, Akq,  Ald, size ), &Ald,
+                           (double*) Mptr( XA,   Akp,   0, XAld, size ), &ione, (double*) one,
+                           (double*) Mptr( YA,     0, Akq, YAld, usiz ), &YAld,
+                           (FC_LEN_T) strlen(TRANS) );
+#else
+/*WCC
                   dagemv_( TRANS, &Amp0, &Anq0, one,
                            Mptr( Aptr, Akp, Akq,  Ald, size ), &Ald,
                            Mptr( XA,   Akp,   0, XAld, size ), &ione, one,
                            Mptr( YA,     0, Akq, YAld, usiz ), &YAld );
+*/
+                  dagemv_( TRANS, &Amp0, &Anq0, (double*) one,
+                           (double*) Mptr( Aptr, Akp, Akq,  Ald, size ), &Ald,
+                           (double*) Mptr( XA,   Akp,   0, XAld, size ), &ione, (double*) one,
+                           (double*) Mptr( YA,     0, Akq, YAld, usiz ), &YAld );
+#endif
                }
             }
          }
@@ -589,8 +660,12 @@ void pdatrmv_( UPLO, TRANS, DIAG, N, ALPHA, A, IA, JA, DESCA, X, IX,
       if( YAsum && ( Amp > 0 ) )
       {
          top = *PB_Ctop( &ctxt, COMBINE, ROW, TOP_GET );
+/*WCC
          Cdgsum2d( ctxt, ROW, &top, Amp, 1, YA, YAd[LLD_], myrow,
                    YAd[CSRC_] );
+*/
+         Cdgsum2d( ctxt, ROW, &top, Amp, 1, (double*) YA, YAd[LLD_], myrow,
+                   YAd[CSRC_] );
       }
    }
    else
@@ -601,8 +676,12 @@ void pdatrmv_( UPLO, TRANS, DIAG, N, ALPHA, A, IA, JA, DESCA, X, IX,
       if( YAsum && ( Anq > 0 ) )
       {
          top = *PB_Ctop( &ctxt, COMBINE, COLUMN, TOP_GET );
+/*WCC
          Cdgsum2d( ctxt, COLUMN, &top, 1, Anq, YA, YAd[LLD_], YAd[RSRC_],
                    mycol );
+*/
+         Cdgsum2d( ctxt, COLUMN, &top, 1, Anq, (double*) YA, YAd[LLD_], YAd[RSRC_],
+                   mycol );
       }
    }
 /*
